/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#include "Assertion.hh"
#include "Node.hh"
#include <limits.h>
#include "expressions/ExpressionLOG.hh"
#include "Values.hh"
#include "Variable.hh"

using namespace epoch::altarica;


std::vector<Variable*> * Assertion::infoVars(const AltaricaModel* am, bool flow, bool computed) const {
  return assert_->infoVars(am, n_, flow, computed);
}

void Assertion::evaluate(const AltaricaModel* am, AltaricaState *s) const {
  Value * v = assert_->evaluate(am, s, n_,true);
  Value::deleteIfPossible(v);
}

void Assertion::evaluateLM(const AltaricaModel*am, LandmarkState *lms) const {
  assert_->evaluateLM(am, lms, n_,true);
}

void Assertion::addVariable(Variable* v) {
	if (variables_ == NULL)
		variables_ = new std::vector<Variable*>();
	// std::cerr << "Adding to assertion variable " << v->getFullName() << std::endl;
	variables_->push_back(v);
}

const  std::vector<Variable*> * Assertion::getVars() const {
  return variables_;
}

Assertion::~Assertion() {
	delete variables_;
}


std::pair<Variable*,Variable*> * Assertion::isSimpleAffect(const AltaricaModel *am) {
  ExpressionLOG * el = dynamic_cast<ExpressionLOG*>(assert_);
  if (el == NULL)
    return NULL;
  return el->isSimpleAffect(am, n_);
}

Assertion * Assertion::clone() const {
  return new Assertion(assert_);
}
