/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#ifndef ALTA_ASSERT_H_
#define ALTA_ASSERT_H_

#include <vector>
#include <stdlib.h>

#include "expressions/Expression.hh"

namespace epoch {
    namespace altarica {

	class Node;
	class Variable;
	class AltaricaState;
	class AltaricaModel;
	class LandmarkState;

	class Assertion {
	protected:
	    Expression * assert_;
	    Node * n_;
	    std::vector<Variable*>* variables_;
	public:
	    Assertion(Expression * a): assert_(a), n_(NULL) {
		    variables_ = NULL;
	    }
	    ~Assertion();

	    std::vector<Variable*> * infoVars(const AltaricaModel *am, bool flow, bool computed) const;

	    void setNode(Node *n) {n_ = n;}
	    const Node * getNode() {return n_;}
	    const Expression * getExpr() const {return assert_;}
	    void addVariable(Variable* a);
	    const  std::vector<Variable*> * getVars() const;

	    void evaluate(const AltaricaModel *am, AltaricaState *s) const;
	    void evaluateLM(const AltaricaModel *am, LandmarkState *lms) const;

	    std::pair<Variable*,Variable*> * isSimpleAffect(const AltaricaModel*am);

	    Assertion * clone() const;

	    void deleteExpr() {delete assert_;}
	};
    }
}

#endif
