/*******************************************************************************
 * Copyright (c) 2015-2016 ONERA.
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *
 *******************************************************************************/

#include "Constant.hh"
#include "./expressions/Expression.hh"

namespace epoch {
  namespace altarica {


    Constant::Constant(std::string name) : expr_(NULL), domain_(NULL) {
      name_ = name;
    }

    Constant::Constant(std::string name, Domain* d) : expr_(NULL) {
      name_ = name;
      domain_ = d;
    }

    Constant::Constant(std::string name, Domain* d, Expression *expr) {
      name_ = name;
      domain_ = d;
      expr_ = expr;
    }

    const Domain* Constant::getDomain() const {
      return domain_;
    }

    const Expression* Constant::getExpression() const {
      return expr_;
    }

    void Constant::setExpression(Expression *expr) {
      delete expr_;
      expr_=expr;
    }

    Value * Constant::getValue(const AltaricaModel *am, AltaricaState *s, const Node* n) const {
      return expr_->evaluate(am, s, n);
    }


    // Constant* Constant::findConstantInVector(std::vector<Constant*> v,
    //                                          std::string *constName) {
    //     for (std::vector<Constant*>::iterator constIt = v.begin();
    //   constIt != v.end(); ++constIt)
    //  if ((*constIt)->getName() == constName)
    //      return *constIt;
    //     return NULL;
    // }

    Constant::~Constant() {
      delete(expr_);
    }

  }
}
