/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#include "ArrayDomain.hh"
#include "Variable.hh"

namespace epoch {
    namespace altarica {
    

	ArrayDomain::ArrayDomain(int asize, Domain*d):asize_(asize), adomain_(d) {
	}
	
	unsigned int ArrayDomain::index(std::string arg) const {
	    BOOST_THROW_EXCEPTION(std::runtime_error(std::string("index asked on a Structured domain")));
	    return 0;	   
	}

	std::string ArrayDomain::text(unsigned int index) const {
	    BOOST_THROW_EXCEPTION(std::runtime_error(std::string("text asked on a Structured domain")));
	    return 0;	   	    
	}

	Value * ArrayDomain::toValue() const {
	    BOOST_THROW_EXCEPTION(std::runtime_error(std::string("toOpenValue asked on a Structured domain")));
	    return NULL;	   	    
	}

	int ArrayDomain::getMin() const {
	    BOOST_THROW_EXCEPTION(std::runtime_error(std::string("getMin asked on an ArrayDomain")));
	    return -1;	   	    
	}

	int ArrayDomain::getMax() const {
	    BOOST_THROW_EXCEPTION(std::runtime_error(std::string("getMax asked on a ArrayDomain")));
	    return -1;	   	    
	}

	// Value * ArrayDomain::getFixedValue(int i) {
	//     BOOST_THROW_EXCEPTION(std::runtime_error(std::string("getFixedValue(i) asked on a ArrayDomain")));
	//     return NULL;	   	    
	// }

	Value * ArrayDomain::getValue(int i) {
	    BOOST_THROW_EXCEPTION(std::runtime_error(std::string("getFixedValue(i) asked on a ArrayDomain")));
	    return NULL;	   	    	    
	}



	std::vector<Variable*> * ArrayDomain::flatten(Variable* v) const {
	    std::vector<Variable*> * nvv = new std::vector<Variable*>();	
	    std::vector<Variable*> * fv = adomain_->flatten(v);
	    for (std::vector<Variable*>::iterator fvi= fv->begin(); fvi != fv->end(); ++fvi) {
		for (int i = 0; i<asize_; ++i) {
		    Variable * nv = (*fvi)->clone();
		    nv->addToMASecond(i);
		    nvv->push_back(nv);
		}		
	    }
	    delete(fv);
	    return nvv;
	}

	std::string ArrayDomain::toString() const {
	    return "ARRAY";
	}
	ArrayDomain::~ArrayDomain() {
	    //	    delete(adomain_);
	}
	
    }
}


