/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#include "ExpressionADD.hh"
#include "Visitor.hh"
#include "ExpressionMUL.hh"

using namespace epoch::altarica;

ExpressionADD * ExpressionADD::clone() const {
    ExpressionADD * nea = new ExpressionADD(left_->clone(), right_->clone());
    if (add_type_ == 0)
	nea->setPlus();
    else
	nea ->setMinus();
    return nea;

}

std::string ExpressionADD::toString() const {
    std::string s;
    s+= left_->toString();
    if (add_type_ == 0)
	s+= " + ";
    else
	s+= " - ";
    s+= right_->toString();
    return s;
}

std::string ExpressionADD::toString(const Node *n, const AltaricaModel& am) const {
    std::string s;
    s+= left_->toString(n, am);
    if (add_type_ == 0)
	s+= " + ";
    else
	s+= " - ";
    s+= right_->toString(n, am);
    return s;
}


std::string ExpressionADD::toString(const Node *n,  const std::pair<std::string,std::string> *f, const AltaricaModel& am) const {
    std::string s;
    s+= left_->toString(n, f, am);
    if (add_type_ == 0)
	s+= " + ";
    else
	s+= " - ";
    s+= right_->toString(n, f, am);
    return s;
}
ExpressionADD::~ExpressionADD() {
    delete left_; 
    delete right_;
}

void ExpressionADD::accept(Visitor* v) const {
    v->visitADD(this);
}
