/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#ifndef EXPRESSION_EVALUATOR_LM_HH
#define EXPRESSION_EVALUATOR_LM_HH

#include "EvaluationVisitor.hh"
#include "../Values.hh"
#include "LandmarkState.hh"

namespace epoch {
    namespace altarica {


	class ExpressionEvaluatorLandmark : public EvaluationVisitor {  
	private :
	    bool assert_;
	    Value * val_;
	    LandmarkState *lms_;
	    landmarkType * lm_;
	public : 


	    ExpressionEvaluatorLandmark(const AltaricaModel *am, LandmarkState *lms, const Node*n, bool assert=false) : EvaluationVisitor(am, lms->getState(),n),assert_(assert), val_(NULL), lms_(lms), lm_(NULL) {};
	    virtual ~ExpressionEvaluatorLandmark() {}
	    Value * getValue() {return val_;}

	    void addOrIntersectLM(landmarkType *lm1, Value *v1, landmarkType*lm2,  Value *v2);
	    void addOrIntersectLM(landmarkType *lm1, Value *v1, AltaricaState* l);
	    void printLM(landmarkType*lm);
	    bool hasLM(landmarkType * lm, Value*v);
	    std::pair<Value*, landmarkType*>  * visitLM(const Expression *e);

	    virtual void visitADD(const ExpressionADD *e);
	    virtual void visitAND(const ExpressionAND *e);
	    virtual void visitARITH(const ExpressionARITH *e)  ;
	    //	    virtual void visitATOMIC(const ExpressionATOMIC *e)  ;
	    virtual void visitBOOLCONSTANT(const ExpressionBOOLCONSTANT *e)  ;
	    virtual void visitCARD(const ExpressionCARD *e)  ;
	    virtual void visitCONSTARRAY(const ExpressionCONSTARRAY *e)  ;
	    virtual void visitCONSTSTRUCT(const ExpressionCONSTSTRUCT *e)  ;
	    virtual void visitCASE(const ExpressionCASE *e)  ;
	    virtual void visitFUNCTIONCALL(const ExpressionFUNCTIONCALL *e)  ;
	    virtual void visitIfThenElse(const ExpressionIfThenElse *e)  ;
	    virtual void visitLOG(const ExpressionLOG *e)  ;
	    virtual void visitMEMBER(const ExpressionMEMBER *e)  ;
	    virtual void visitMINMAX(const ExpressionMINMAX *e)  ;
	    virtual void visitMUL(const ExpressionMUL *e)  ;
	    virtual void visitOR(const ExpressionOR *e)  ;
	    virtual void visitPAREN(const ExpressionPAREN *e)  ;
	    virtual void visitQUANTIFIED(const ExpressionQUANTIFIED *e)  ;
	    virtual void visitUNSIGNEDINT(const ExpressionUNSIGNEDINT *e)  ;
	    virtual void visitREAL(const ExpressionREAL *e)  ;
	    virtual void visitUNARY(const ExpressionUNARY *e)  ;
	    virtual void visitINTERVAL(const ExpressionINTERVAL *e)  ;

	private:
	    std::vector<std::vector<Value*>*> * buildCombinations(std::vector<Variable*> * vars);
	};
    }
}

#endif
