/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#ifndef ALTA_EXPRESSION_IFTHENELSE_H_
#define ALTA_EXPRESSION_IFTHENELSE_H_


#include "Expression.hh"



namespace epoch {
    namespace altarica {


	class ExpressionIfThenElse : public Expression {
	
	private:
	    
	    Expression *cond_;	    
	    Expression *then_;
	    Expression *else_;

	public:
 	    ExpressionIfThenElse(): cond_(NULL), then_(NULL), else_(NULL) {}
 	    virtual ~ExpressionIfThenElse() { delete cond_;  delete then_;   delete else_;}
 	    ExpressionIfThenElse(Expression *i, Expression *t) : Expression(), cond_(i), then_(t), else_(NULL) {}
 	    ExpressionIfThenElse(Expression *i, Expression *t, Expression *e) : Expression(), cond_(i), then_(t), else_(e) {}
	    

	    virtual ExpressionIfThenElse * clone() const {return new ExpressionIfThenElse(cond_, then_, else_);}

	    virtual void accept(Visitor* v) const {v->visitIfThenElse(this);}
	    virtual std::string toString() const;
	    virtual std::string toString(const Node *n, const AltaricaModel& am) const;
	    virtual std::string toString(const Node *n, const std::pair<std::string,std::string> *f, const AltaricaModel& am) const;
	    virtual std::string toFiacre(const Node *n, const std::pair<std::string,std::string> *f, const AltaricaModel& am, std::string head) const;
	    Expression *getIf() const {return cond_;}
	    Expression *getThen() const {return then_;}
	    Expression *getElse() const {return else_;}
	};
    }
}


#endif
