/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#ifndef EXPRESSION_INFOVAR_HH
#define EXPRESSION_INFOVAR_HH

#include "EvaluationVisitor.hh"
#include <vector>


namespace epoch {
    namespace altarica {
	
	class Node;
	class Variable;
	class AltaricaState;
	class AltaricaModel;

	class ExpressionInfoVar : public EvaluationVisitor {
	    
	private:
	    const Node *n_;
	    bool flow_;
	    bool computed_;
	    std::vector<Variable*> * vars_;
	    
	    
	public:
	    ExpressionInfoVar(bool flow, bool computed, const AltaricaModel *am, AltaricaState *s, const Node*n);
	    std::vector<Variable*> * getVars() {return vars_;}
	    virtual ~ExpressionInfoVar() {}
	    
	    virtual void visitADD(const ExpressionADD *e) ;
	    virtual void visitAND(const ExpressionAND *e) ;
	    virtual void visitARITH(const ExpressionARITH *e) ;
	    //	    virtual void visitATOMIC(const ExpressionATOMIC *e) ;
	    virtual void visitBOOLCONSTANT(const ExpressionBOOLCONSTANT *e) ;
	    virtual void visitCARD(const ExpressionCARD *e) ;
	    virtual void visitCONSTARRAY(const ExpressionCONSTARRAY *e) ;
	    virtual void visitCONSTSTRUCT(const ExpressionCONSTSTRUCT *e) ;
	    virtual void visitCASE(const ExpressionCASE *e) ;
	    virtual void visitFUNCTIONCALL(const ExpressionFUNCTIONCALL *e) ;
	    virtual void visitIfThenElse(const ExpressionIfThenElse *e) ;
	    virtual void visitLOG(const ExpressionLOG *e) ;
	    virtual void visitMEMBER(const ExpressionMEMBER *e) ;
	    virtual void visitMINMAX(const ExpressionMINMAX *e) ;
	    virtual void visitMUL(const ExpressionMUL *e) ;
	    virtual void visitOR(const ExpressionOR *e) ;
	    virtual void visitPAREN(const ExpressionPAREN *e) ;
	    virtual void visitQUANTIFIED(const ExpressionQUANTIFIED *e) ;
	    virtual void visitUNSIGNEDINT(const ExpressionUNSIGNEDINT *e) ;
	    virtual void visitREAL(const ExpressionREAL *e) ;
	    virtual void visitUNARY(const ExpressionUNARY *e) ;
	    virtual void visitINTERVAL(const ExpressionINTERVAL *e) ;

	};

    }
}

#endif
