/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#include "ExpressionMUL.hh"
#include "ExpressionUNARY.hh"

using namespace epoch::altarica;


ExpressionMUL * ExpressionMUL::clone() const {
    ExpressionMUL * nem = new ExpressionMUL(left_->clone(), right_->clone());
    switch(mul_type_) {
    case 0: nem->setTimes(); break;
    case 1: nem->setDiv(); break;
    case 2: nem->setMod(); break;
    }
    return nem;
}

std::string ExpressionMUL::toString() const {
    std::string s;
    s+= left_->toString();
    switch (mul_type_) {
    case 0: s+= "*"; break;
    case 1: s+= "/"; break;
    case 2: s+= "%"; break;
    }
    s+= right_->toString();
    return s;
}

std::string ExpressionMUL::toString(const Node *n, const AltaricaModel& am) const {
    std::string s;
    s+= left_->toString(n, am);
    switch (mul_type_) {
    case 0: s+= "*"; break;
    case 1: s+= "/"; break;
    case 2: s+= "%"; break;
    }
    s+= right_->toString(n, am);
    return s;
}

std::string ExpressionMUL::toString(const Node *n, const std::pair<std::string,std::string> *f, const AltaricaModel& am) const {
    std::string s;
    s+= left_->toString(n, f, am);
    switch (mul_type_) {
    case 0: s+= "*"; break;
    case 1: s+= "/"; break;
    case 2: s+= "%"; break;
    }
    s+= right_->toString(n, f, am);
    return s;
}

ExpressionMUL::~ExpressionMUL() {
    delete left_; 
    delete right_;
}
