/*******************************************************************************
 * Copyright (c) 2015-2017 ONERA and IRT AESE (IRT Saint Exupéry).
 * Ce logiciel est la propriété de l’ONERA (the French Aerospace Lab) et de
 * l'IRT AESE (IRT Saint Exupéry).
 * Tous droits réservés.
 *
 * Ce programme et les éléments qui l'accompagnent sont mis à disposition
 * aux conditions définies par le contrat de licence logiciel CeCILL-C soumise
 * au droit français et respectant les principes de diffusion des logiciels libres.
 * Vous pouvez utiliser, modifier et/ou redistribuer ce programme
 * sous les conditions de la licence CeCILL-C (http://www.cecill.info).

 * This software is property of ONERA (the French Aerospace Lab) and of
 * the IRT AESE (IRT Saint Exupéry).
 * All rights reserved.
 *
 * This program and the accompanying materials are made available under
 * the terms of the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.
 * You can  use, modify and/ or redistribute the software under the terms of
 * the CeCILL-C license (http://www.cecill.info).
 *
 * Contributeurs/contributors:
 *     Guillaume Infantes (ONERA - Centre de Toulouse) - initial API and implementation
 *     Alexandre Albore (IRT Saint-Exupéry) - Altarica/Fiacre translation
 *
 *******************************************************************************/

#include "ExpressionQUANTIFIED.hh"
#include "Visitor.hh"

using namespace epoch::altarica;

ExpressionQUANTIFIED* ExpressionQUANTIFIED::clone() const {
    ExpressionQUANTIFIED * neq = new ExpressionQUANTIFIED(forAll_);
    neq->setExpr(expr_->clone());
    std::vector<std::string>::const_iterator si = qVars_.begin();
    std::vector<Domain*>::const_iterator di = dVars_.begin();
    for (; si!= qVars_.end(); ++si, ++di)
	neq->addVar(*si,*di);
    return neq;
}

std::string ExpressionQUANTIFIED::toString() const {
    std::string s;
    if (forAll_)
	s+= "forAll (";
    else
	s+= "exists (";
    std::vector<std::string>::const_iterator vi = qVars_.begin();
    std::vector<Domain*>::const_iterator di = dVars_.begin();
    for (; vi!= qVars_.end(); ++vi, ++di)
	s+= *vi + " in " + (*di)->toString() + ",";
    s+= " : " + expr_->toString();
    return s;
}

std::string ExpressionQUANTIFIED::toString(const Node *n, const AltaricaModel& am) const {
    std::string s;
    if (forAll_)
	s+= "forAll (";
    else
	s+= "exists (";
    std::vector<std::string>::const_iterator vi = qVars_.begin();
    std::vector<Domain*>::const_iterator di = dVars_.begin();
    for (; vi!= qVars_.end(); ++vi, ++di)
	s+= *vi + " in " + (*di)->toString() + ",";
    s+= " : " + expr_->toString(n, am);
    return s;
}

std::string ExpressionQUANTIFIED::toString(const Node *n, const std::pair<std::string,std::string> *f, const AltaricaModel& am) const {
    std::string s;
    if (forAll_)
	s+= "forAll (";
    else
	s+= "exists (";
    std::vector<std::string>::const_iterator vi = qVars_.begin();
    std::vector<Domain*>::const_iterator di = dVars_.begin();
    for (; vi!= qVars_.end(); ++vi, ++di)
	s+= *vi + " in " + (*di)->toString() + ",";
    s+= " : " + expr_->toString(n, f, am);
    return s;
}
